#ifndef EDGE_DATABASE_H
#define EDGE_DATABASE_H

#include <stdint.h>
#include <vector>
#include <cassert>
#include <tcutil.h>
#include <tchdb.h>
#include "maxlink_types.h"
#include "hdb_tokyo_cabinet_database.h"

class EdgeDatabase : public HdbTokyoCabinetDatabase {
 public:
  EdgeDatabase() {};
  ~EdgeDatabase() {};
  void storeEdge(maxlink_types::GeneId gene_id1, maxlink_types::GeneId gene_id2, maxlink_types::Pfc pfc, maxlink_types::Fbs fbs) {
    assert(init_run_);

    maxlink_types::HalfEdge edge;
    edge.gene_id = gene_id2;
    edge.pfc = pfc;
    edge.fbs = fbs;

    assert(gene_id1 != gene_id2);

    assert(hdb_);
    if (!tchdbputcat(hdb_, &gene_id1, sizeof(gene_id1), &edge, sizeof(edge))) {
      int error_code = tchdbecode(hdb_);
      fprintf(stderr, "error in tchdbputcat: %s\n", tchdberrmsg(error_code));
    }
    edge.gene_id = gene_id1;
    if (!tchdbputcat(hdb_, &gene_id2, sizeof(gene_id2), &edge, sizeof(edge))) {
      int error_code = tchdbecode(hdb_);
      fprintf(stderr, "error in tchdbputcat: %s\n", tchdberrmsg(error_code));
    }
  }

  std::string endingFilename() {
    return std::string("EdgeDatabase.hdb");
  };

  std::vector<maxlink_types::GeneId> neighbours(bool debug, maxlink_types::GeneId gene_id, maxlink_types::Fbs cutoff) const {
   /* 
      At first sight the function: 

        int tchdbget3(TCHDB *hdb, const void *kbuf, int ksiz, void *vbuf, int max);

      seems to be useful, but at first sight it doesn't seem to distinguish between the case that max is
      to small and that the key is missing.

    */
    assert(init_run_);
    if (debug) {
      std::cerr << "debug: in function EdgeDatabase::neighbours" << std::endl;
      std::cerr << "debug: gene id = " << gene_id << " fbs cutoff = " << cutoff << std::endl;
    }

     int value_size = 0;
     std::vector<maxlink_types::GeneId> vec;
     maxlink_types::HalfEdge *result_ptr = static_cast<maxlink_types::HalfEdge *>(tchdbget(hdb_, &gene_id, sizeof(gene_id), &value_size));
     if (!result_ptr) {
       return vec;
     }
     // TODO: use this technique instead
    // http://stackoverflow.com/questions/259297/how-do-you-copy-the-contents-of-an-array-to-a-stdvector-in-c-without-looping
     maxlink_types::HalfEdge *value_ptr = result_ptr;
     for (int i = 0; i < value_size / sizeof(maxlink_types::HalfEdge); ++i) {
       if (cutoff < value_ptr->fbs) {
         vec.push_back(value_ptr->gene_id);
       }
       ++value_ptr;
     }
     free(result_ptr);
     return vec;
   }
};

#endif
