#ifndef MAXLINK_GENE_LIST_XML_PARSER_H
#define MAXLINK_GENE_LIST_XML_PARSER_H

#include <vector>
#include <memory>

#include "maxlink_gene_list.h"
#include "maxlink_types.h"
#include "string2num_gene_database.h"
#include "config.h"

class MaxlinkGeneListXmlParser {
 public:
  MaxlinkGeneListXmlParser() {};
  ~MaxlinkGeneListXmlParser() {};
  std::vector<maxlink_types::GeneId> loadGeneVectorFromXml(const char *xml_filename, String2NumGeneDatabase *string2num_gene_database) {
    std::vector<maxlink_types::GeneId> gene_vector;
    try {
      xml_schema::properties props;
      props.schema_location(MAXLINK_GENE_LIST_NAMESPACE, MAXLINK_GENE_LIST_SCHEMA_INSTALL_PATH);
      std::auto_ptr<maxlink_gene_list_ns::maxlink_gene_list> root(maxlink_gene_list_ns::maxlink_gene_list_(xml_filename, 0, props));
      BOOST_FOREACH(const maxlink_gene_list_ns::maxlink_gene_list::gene_type &gene, root->gene()) {
       maxlink_types::GeneId gene_id;
       bool ret = string2num_gene_database->translateString2Num(gene, &gene_id);
       if (ret == false) {
         std::cerr << "The gene id \"" << gene << "\" was not found" << std::endl;
         exit(EXIT_FAILURE);
       }
       gene_vector.push_back(gene_id);
      }
    }
    catch(const xml_schema::exception &e) {
      std::cerr << e << std::endl;
      exit(EXIT_FAILURE);
    }
    catch(const std::ios_base::failure &) {
      std::cerr << ": unable to open or read/write failure" << std::endl;
      exit(EXIT_FAILURE);
    }
    catch(...) {
      std::cerr << "unknown exception" << std::endl;
      exit(EXIT_FAILURE);
    }
    return gene_vector;
  }
};

#endif
